@@ -46,7 +46,7 @@ OF_ASSUME_NONNULL_BEGIN
 #ifdef __cplusplus
 extern "C" {
 #endif
-extern int _OFUTF8StringCheck(const char *, size_t, size_t *)
+extern int _OFUTF8StringCheck(const char *, size_t, size_t *_Nullable)
     OF_VISIBILITY_HIDDEN;
 extern size_t _OFUTF8StringIndexToPosition(const char *, size_t, size_t)
     OF_VISIBILITY_HIDDEN;
