@@ -28,7 +28,7 @@ OF_ASSUME_NONNULL_BEGIN
 #ifdef __cplusplus
 extern "C" {
 #endif
-extern int _NSSet_OFObject_reference;
+extern int _NSSet_OFObject_reference OF_VISIBILITY_HIDDEN;
 #ifdef __cplusplus
 }
 #endif
